#!/home/nickless/Netstuff/catchnetflow/private-perl/linux/bin/perl -w

# released under the Msys Public License.  See license.txt for details,
# or http://www-unix.mcs.anl.gov/systems/software/msys/copyright.txt .

use DBI;
use IO::Socket;
use IO::Handle;

# Convert a UNIX time value into something that we can shove into Oracle

sub timestr {
	my $t=shift(@_);
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime($t);
	$mon++; # localtime() returns 0=Jan, 1=Feb, etc
	"$mon/$mday/$year-$hour:$min:$sec";
}

# Convert an IP address in integer format to a decimal number.  That is,
# IP address 140.221.9.6 is 0x8cdd0906, and we want this function to
# return 140221009006.

sub num2ipnum {
	my $a=shift(@_);
	my $a1=($a&0xff000000)>>24;
    my $a2=($a&0x00ff0000)>>16;
    my $a3=($a&0x0000ff00)>>8;
    my $a4=($a&0x000000ff);
    sprintf("%3.3d%3.3d%3.3d%3.3d",$a1,$a2,$a3,$a4);
}

#
# Set up connections and statements for oracle
#

$ENV{'ORACLE_USERID'}="nickless/password";

$dbh = DBI->connect("DBI:Oracle:db02",$ENV{'ORACLE_USERID'},"") ||
    die $DBI::errstr;

$dbh->{AutoCommit}=0;
$dbh->{PrintError}=1;

$datst = $dbh->prepare("alter session set nls_date_format = ".
        "'MM/DD/YY,HH24:MI:SS'");
$datst->execute || die $DBI::errstr;

$inxst = $dbh->prepare("alter session set skip_unusable_indexes = true");
$inxst->execute || die $DBI::errstr;

$fli=$dbh->prepare
    ("insert into cflow.flow2".
     "(router_id,src_ipn,dst_ipn,nxt_ipn,ifin,ifout,packets,octets,starttime,".
     "endtime,srcport,dstport,tcp,prot,tos,srcas,dstas,srcmask,dstmask)".
     "values (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");

$rts=$dbh->prepare("select * from cflow.routers");
$rts->execute();
while(@row = $rts->fetchrow_array) {
	printf("$row[3] will refer to $row[0] will refer to $row[2]\n");

	$routers{$row[3]}=$row[0];
    $routername{$row[0]}=$row[2];
}

printf ("IP address is %12.12g\n",num2ipnum(0x0cdd0906));
printf ("IP address is %12.12g\n",num2ipnum(0x0c000000));

#
# Actual program starts here
#

select((select(STDOUT),$|=1)[0]);

# Open a socket to receive Cisco netflow output packets

my $flows = IO::Socket::INET->new(
				Proto		=> "udp",
				LocalAddr	=> "0.0.0.0",
				LocalPort	=> "(2155)",
			)
			or die "cannot open socket to listen on 2155";

# Loop and handle them

FOREVER: while(1) {
	my ($buf,$rcvval);
	$rcvval=$flows->recv($buf,2000,0) || die "recv: $!";
    my $rtrkey=$routers{num2ipnum(unpack('x4 N x8',$rcvval))};
#	printf "Received %d byte packet\n",length($buf);
#	printf "  Router address       : %s\n",
#             num2ipnum(unpack('x4 N x8',$rcvval));
#	printf "  Router Name          : %s\n",$routername{$rtrkey};

	if (!($rtrkey)) {
		printf "UNKNOWN ROUTER ";
        next FOREVER;
    }
	
	my @FLOWS=unpack("a24 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48".
					" a48 a48 a48 a48 a48 a48 a48 a48 a48 a48 a48".
					" a48 a48 a48 a48 a48 a48 a48 a48",$buf); 

	my ($hver,$hcount,$hsysuptime,$hunix_secs,$hunix_nsecs,$hflow_sequence)=
		unpack("n n N N N N x4",shift(@FLOWS));

	next FOREVER if ($hver != 5);
	next FOREVER if ($hcount != ((length($buf)-24)/48));

	# Figure out the time that the router was rebooted, since that is what
    # the sysUptime value in each flow will be based on
	my $basetime=$hunix_secs+($hunix_nsecs/1000000000)-($hsysuptime/1000);

#	printf "   OK packet, flows    : %d\n",$hcount;

    HANDLE_A_PACKET: 
    while(1) {
		my $thisflow=shift(@FLOWS);
		last HANDLE_A_PACKET 
            if ((!defined($thisflow))||length($thisflow)<48);
		my ($fsrc,$fdst,$fnext,$fin,$fout,$fpkts,$focts,$fstrt,$fend,$fsrcp,
			$fdstp,$ftcp,$fprot,$ftos,$fsas,$fdas,$fsmsk,$fdmsk)=
			unpack("N3 n2 N4 n2 x C3 n2 C2 x2",$thisflow);
#		printf "    %s/%d -> %s/%d\n",
#			inet_ntoa($fsrc),$fsmsk,inet_ntoa($fdst),$fdmsk;
#		printf "    %s %s\n",
#			timestr($basetime+($fstrt/1000)),
#			timestr($basetime+($fend/1000));

        $fli->execute($rtrkey,
                      num2ipnum($fsrc),
                      num2ipnum($fdst),
                      num2ipnum($fnext),
                      sprintf("%4.4lx",$fin),
                      sprintf("%4.4lx",$fout),
                      $fpkts,
                      $focts,
                      timestr($basetime+($fstrt/1000)),
                      timestr($basetime+($fend/1000)),
                      sprintf("%4.4lx",$fsrcp),
                      sprintf("%4.4lx",$fdstp),
                      sprintf("%2.2lx",$ftcp),
                      sprintf("%2.2lx",$fprot),
                      sprintf("%2.2lx",$ftos),
                      sprintf("%4.4lx",$fsas),
                      sprintf("%4.4lx",$fdas),
                      sprintf("%2.2lx",$fsmsk),
                      sprintf("%2.2lx",$fdmsk)) || die $DBI::errstr;

    }
	printf ("%d ",$hcount);
    $dbh->commit;
}
